﻿using Foundation;
using UIKit;
using AsReaderGun251GSDK;
using System;
using MBProgressHud;
using System.Collections.Generic;
using CoreAnimation;

namespace AsReaderGun251GDemoApp
{
    // The UIApplicationDelegate for the application. This class is responsible for launching the
    // User Interface of the application, as well as listening (and optionally responding) to application events from iOS.
    [Register("AppDelegate")]
    public class AppDelegate : UIResponder, IUIApplicationDelegate
    {
        public int kBluetoothAlertView = 0;
        public int kDeviceAlertView = 1;
        public int VersionAlertView = 2;
        public int VersionForceAlertView = 3;

        public string[] DATA_TARGET = { "S0", "S1", "S2", "S3", "SL" };
        public string[] DATA_ACTION = { "AB", "AN", "NB", "MN", "BA", "BN", "NA", "NM" };
        public string[] DATA_BANK = { "Reserved", "EPC", "TID", "User" };
        public string[] DATA_BIT = { "0", "16", "32", "48", "64", "80", "96", "112", "128", "144", "160", "176", "192", "208", "224", "240" };
        public string[] DATA_WORD = { "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15" };
        public string[] DATA_SELECT_FLAG = { "SL", "~SL", "All" };
        public string[] DATA_INVENTORY_SESSION = { "S0", "S1", "S2", "S3" };
        public string[] DATA_SESSION_FLAG = { "A Only", "B Only", "A or B" };

        public int MAX_PASSWORD_LENGTH = 8;
        public int MAX_WRITE_DATA_WORD = 32;
        public int MAX_WRITE_DATA_LENGTH = 32 * 4;

        readonly SystemSetting systemSetting = SystemSetting.sharedSystemSetting();

        [Export("window")]
        public UIWindow Window { get; set; }

        [Export("application:didFinishLaunchingWithOptions:")]
        public bool FinishedLaunching(UIApplication application, NSDictionary launchOptions)
        {
            // Override point for customization after application launch.
            // If not required for your application you can safely delete this method

            if (!systemSetting.getFirstUsed())
            {
                systemSetting.setTidOn(false);
                systemSetting.setTidLength(4);
                systemSetting.setFirstUser(true);
                systemSetting.setBuzzer((int)BuzzerState.High);
                systemSetting.setVibrator((int)VibratorState.n);
                systemSetting.setInventoryTime(400);
                systemSetting.setIdleTime(100);
                systemSetting.setAutoOffTime(600);
                systemSetting.setQValue(4);
                systemSetting.setAlgorithm((int)AlgorithmType.DynamicQ);
                systemSetting.setMaxQ(15);
                systemSetting.setMinQ(0);
                systemSetting.setSleepTime(300);
                systemSetting.setMaskTypeValue((int)MaskType.NoMask);
                systemSetting.setLinkProfileValue(3);
                systemSetting.setDefaultLinkProfileValue(1);
                systemSetting.setOperationTime(0);
                systemSetting.setPowerGain(300);
                systemSetting.setInventorySession((int)SessionType.Session_S0);
                systemSetting.setSessionFlag(SessionFlag.Ab);
                systemSetting.setContinuousMode(true);
                systemSetting.setRssiMode(false);
                systemSetting.setPropBarcodeMode(true);
                systemSetting.setEpcMaskMatchMode(false);
                systemSetting.setDisPlayPC(true);
                systemSetting.setLbtItemData(null);
                systemSetting.setEpcMaskData(null);
                systemSetting.setUseSelectionMask(SelectFlag.All);
                systemSetting.setSelectionEpcMaskData(null);
                systemSetting.setBarcodeParamData(null);
                systemSetting.setBuzzerTime(1);
                systemSetting.setVibratorTime(1);
                systemSetting.setBarcodeReScanTime(0);
                systemSetting.setCustomPrefixSuffix(false);
            }
            NSNotificationCenter center = NSNotificationCenter.DefaultCenter;
            NSString connectedStr = new NSString("Alert");
            center.AddObserver(connectedStr, showAlert);
            return true;
        }

        void showAlert(NSNotification notification)
        {
            NSDictionary dic = notification.UserInfo;
            if (dic != null)
            {
                NSString codeString = (NSString)dic.ObjectForKey(new NSString("DATA"));
                UIAlertController alert = UIAlertController.Create("Alert", codeString, UIAlertControllerStyle.Alert);
                alert.AddAction(UIAlertAction.Create("OK", UIAlertActionStyle.Cancel, null));
                UIWindow window = UIApplication.SharedApplication.KeyWindow;
                window.RootViewController.PresentViewController(alert, true, null);
            }
        }

        // UISceneSession Lifecycle

        [Export("application:configurationForConnectingSceneSession:options:")]
        public UISceneConfiguration GetConfiguration(UIApplication application, UISceneSession connectingSceneSession, UISceneConnectionOptions options)
        {
            // Called when a new scene session is being created.
            // Use this method to select a configuration to create the new scene with.
            return UISceneConfiguration.Create("Default Configuration", connectingSceneSession.Role);
        }

        [Export("application:didDiscardSceneSessions:")]
        public void DidDiscardSceneSessions(UIApplication application, NSSet<UISceneSession> sceneSessions)
        {
            // Called when the user discards a scene session.
            // If any sessions were discarded while the application was not running, this will be called shortly after `FinishedLaunching`.
            // Use this method to release any resources that were specific to the discarded scenes, as they will not return.
        }

        //#pragma mark - Utility Methods
        public string GetAppVersion()
        {
            NSDictionary infoDictionary = NSBundle.MainBundle.InfoDictionary;
            NSString majorVersion = (NSString)infoDictionary.ObjectForKey(new NSString("CFBundleShortVersionString"));
            NSString minorVersion = (NSString)infoDictionary.ObjectForKey(new NSString("CFBundleVersion"));
            return "Version " + majorVersion.ToString() + " Build " + minorVersion.ToString();
        }

        public UIViewController GetCurrentVC()
        {
            UIViewController result;
            UIWindow window = UIApplication.SharedApplication.KeyWindow;
            if (window.WindowLevel == UIWindowLevel.Normal)
            {
                Array windows = UIApplication.SharedApplication.Windows;
                foreach (UIWindow tmpWin in windows)
                {
                    if (tmpWin.WindowLevel == UIWindowLevel.Normal)
                    {
                        window = tmpWin;
                        break;
                    }
                }

            }
            UIView frontView = window.Subviews[0];
            var nextResponder = frontView.NextResponder;

            UIViewController controller = new UIViewController();
            if (nextResponder.Class == controller.Class)
                result = (UIViewController)nextResponder;
            else
                result = window.RootViewController;
            return result;
        }
        public static void showGlobalProgressHUDWithTitle(string title)
        {
            UIWindow window = UIApplication.SharedApplication.KeyWindow;
            MBProgressHUD hud = MBProgressHUD.ShowHUDAddedTo(window, true);
            hud.LabelText = title;
            hud.Mode = MBProgressHUDMode.Text;
            CATransaction.Commit();
        }
        public static void dismissGlobalHUD()
        {
            UIWindow window = UIApplication.SharedApplication.KeyWindow;
            MBProgressHUD.HideAllHUDsForView(window, true);
            CATransaction.Commit();
        }
    }
}

