﻿using System;
using Foundation;
using AsReaderGun251GSDK;
using UIKit;

namespace AsReaderGun251GDemoApp
{
    public class AsReaderGUNManager:NSObject
    {
        public delegate void AccessResult(ResultCode error, CommandType action, string epc, string data, float rssi, float phase, float frequency);
        public delegate void DetectBarcode(BarcodeType barcodeType, string codeId, string barcode);
        public delegate void DetectBarcodeData(BarcodeType barcodeType, string codeId, NSData barcodeData);
        public delegate void CommandComplete(CommandType command);
        public delegate void ChangedActionState(CommandType action, nint resultCode);
        public delegate bool OnAsReaderTriggerKeyEvent(bool status);
        public delegate bool OnAsReaderLeftModeKeyEvent(bool status);
        public delegate bool OnAsReaderRightModeKeyEvent(bool status);
        public delegate void ReaderInitialized(AsReader reader);
        public delegate void UpdateDeviceState(ResultCode error);
        public delegate void ReadTag(string tag, float rssi, float phase, float frequency);
        public delegate void OnAsReaderGUNConnected();
        public delegate void OnAsReaderGUNDisconnected();
        public delegate void OnModuleOverHeated();
        public delegate void OnResetReader();
        public delegate void ReadTagWithTid(ResultCode error, CommandType action, string epc, string data, float rssi, float phase, float frequency);
        public delegate void ReadTagWithRFM(ResultCode error, CommandType action, string epc, float rssi, float phase, float frequency, string tid, string tempCode, string calibration, string sensorCode);

        public AccessResult accessResult;
        public DetectBarcode detectBarcode;
        public DetectBarcodeData detectBarcodeData;
        public CommandComplete commandComplete;
        public ChangedActionState changedActionState;
        public OnAsReaderTriggerKeyEvent onAsReaderTriggerKeyEvent;
        public OnAsReaderLeftModeKeyEvent onAsReaderLeftModeKeyEvent;
        public OnAsReaderRightModeKeyEvent onAsReaderRightModeKeyEvent;
        public ReaderInitialized readerInitialized;
        public UpdateDeviceState updateDeviceState;
        public ReadTag readTag;
        public OnAsReaderGUNConnected onAsReaderGUNConnected;
        public OnAsReaderGUNDisconnected onAsReaderGUNDisconnected;
        public OnModuleOverHeated onModuleOverHeated;
        public OnResetReader onResetReader;
        public ReadTagWithTid readTagWithTid;
        public ReadTagWithRFM readTagWithRFM;

        static AsReaderGUNManager uniqueInstance;
        static readonly object locker = new object();

        public AsReaderGUN mAsReaderGUN;
        static AsReader mReader;
        static MyAsreaderDelegate mAsreaderDelegate;

        SelectFlag backupUseMask;
        readonly NSMutableArray backupSelectMasks = new NSMutableArray();

        public static AsReaderGUNManager SharedAsReaderGUNManager()
        {
            if (uniqueInstance == null)
            {
                lock (locker)
                {
                    if (uniqueInstance == null)
                    {
                        uniqueInstance = new AsReaderGUNManager();
                    }
                }
            }
            return uniqueInstance;
        }
        public AsReaderGUNManager()
        {
            mAsReaderGUN = new AsReaderGUN("com.asreader.gun");

            mAsreaderDelegate = new MyAsreaderDelegate(this);

            NSNotificationCenter center = NSNotificationCenter.DefaultCenter;
            NSString connectedStr = new NSString("AsReaderGUNConnected");
            NSString disconnectedStr = new NSString("AsReaderGUNDisconnected");

            center.AddObserver(connectedStr, AsReaderGUNConnected);
            center.AddObserver(disconnectedStr, AsReaderGUNDisconnected);
        }

        public AsReader GetCurrentAsReader()
        {
            return mReader;
        }

        public void ResetDelegate()
        {
            accessResult = null;
            detectBarcode = null;
            commandComplete = null;
            changedActionState = null;
            onAsReaderTriggerKeyEvent = null;
            onAsReaderLeftModeKeyEvent = null;
            onAsReaderRightModeKeyEvent = null;
            readerInitialized = null;
            updateDeviceState = null;
            readTag = null;
            onAsReaderGUNDisconnected = null;
            onModuleOverHeated = null;
            onResetReader = null;
        }

        public void InitMask(string maskValue, MaskType maskType)
        {
            if (maskValue != null)
            {
                if (maskType == MaskType.Selection)
                {
                    backupUseMask = mReader.UseSelectionMask;

                    for (int i = Define.MIN_SELECTION_MASK; i < Define.MAX_SELECTION_MASK; i++)
                    {
                        if (!mReader.UsedSelectionMask(i)) break;

                        AsSelectMaskParam maskParam = mReader.GetSelectionMask(i);
                        backupSelectMasks.Add(maskParam);
                    }
                }
                AsSelectMaskParam param = new AsSelectMaskParam(0, MaskTargetType.L, MaskActionType.Ab, MemoryBank.Epc, 32, maskValue.Substring(4), true);
                mReader.SetSelectionMask(0, param);
                mReader.UseSelectionMask = SelectFlag.Sl;

                for (int i = 1; i < Define.MAX_SELECTION_MASK; i++)
                    mReader.SetMaskUsed(i, false);
            }

        }

        public void ExitMask()
        {
            mReader.UseSelectionMask = backupUseMask;

            for (int i = Define.MIN_SELECTION_MASK; i < Define.MAX_SELECTION_MASK; i++)
            {
                if ((int)backupSelectMasks.Count < i + 1)
                {
                    mReader.RemoveSelectionMask(i);
                    break;
                }

                AsSelectMaskParam param = backupSelectMasks.GetItem<AsSelectMaskParam>((nuint)i);

                if (param == null) break;

                mReader.SetSelectionMask(i, param);

            }
            backupSelectMasks.RemoveAllObjects();
            //EPC MASK Rollback
            if ((MaskType)mReader.MaskTypeValue == MaskType.Epc)
            {
                rollBackEpcMask();
            }
        }

        private void rollBackEpcMask()
        {
            int count = mReader.EpcMaskCount;
            Console.WriteLine(" EPC MASK Count : " + count);
            AsSelectMaskEPCParam param;
            NSMutableArray maskData = new NSMutableArray();
            for (int i = 0; i < count; i++)
            {
                param = mReader.GetEpcMask(i);
                if (param != null)
                {
                    maskData.Add(param);
                }
            }
            ResultCode resultCD = mReader.ClearEpcMask;
            for (nuint i = 0; i < maskData.Count; i++)
            {
                AsSelectMaskEPCParam epcParam = maskData.GetItem<AsSelectMaskEPCParam>(i);
                mReader.AddEpcMask(epcParam);
            }
            if (mReader.EpcMaskMatchMode)
            {
                mReader.EpcMaskMatchMode = true;
            } else {
                mReader.EpcMaskMatchMode = false;
            }
            count = mReader.EpcMaskCount;
            Console.WriteLine(" EPC MASK Count : " + count);
        }

        class MyAsreaderDelegate : AsReaderDelegate
        {
            readonly AsReaderGUNManager gunManager;
            public MyAsreaderDelegate(AsReaderGUNManager _gunManager)
            {
                gunManager = _gunManager;
            }

            public override void AccessResult(ResultCode error, CommandType action, string epc, string data, float rssi, float phase, float frequency)
            {
                gunManager.accessResult?.Invoke(error, action, epc, data, rssi, phase, frequency);
            }
            public override void DetectBarcode(BarcodeType barcodeType, string codeId, string barcode)
            {
                gunManager.detectBarcode?.Invoke(barcodeType, codeId, barcode);
            }

            public override void DetectBarcode(BarcodeType barcodeType, string codeId, NSData barcodeData)
            {
                gunManager.detectBarcodeData?.Invoke(barcodeType, codeId, barcodeData);
            }
            public override void CommandComplete(CommandType command)
            {
                gunManager.commandComplete?.Invoke(command);
            }
            public override void ChangedActionState(CommandType action, nint resultCode)
            {
                gunManager.changedActionState?.Invoke(action, resultCode);
            }
            public override bool OnAsReaderTriggerKeyEvent(bool status)
            {
                if (gunManager.onAsReaderTriggerKeyEvent != null) return gunManager.onAsReaderTriggerKeyEvent(status);
                return false;
            }
            public override bool OnAsReaderLeftModeKeyEvent(bool status)
            {
                if (gunManager.onAsReaderLeftModeKeyEvent != null) return gunManager.onAsReaderLeftModeKeyEvent(status);
                return false;
            }
            public override bool OnAsReaderRightModeKeyEvent(bool status)
            {
                if (gunManager.onAsReaderRightModeKeyEvent != null) return gunManager.onAsReaderRightModeKeyEvent(status);
                return false;
            }
            public override void ReaderInitialized(AsReader reader)
            {
                gunManager.readerInitialized?.Invoke(reader);
            }

            public override void UpdateDeviceState(ResultCode error)
            {
                gunManager.updateDeviceState?.Invoke(error);
            }

            public override void ReadTag(string tag, float rssi, float phase, float frequency)
            {
                gunManager.readTag?.Invoke(tag, rssi, phase, frequency);
            }
            public override void OnModuleOverHeated()
            {
                gunManager.onModuleOverHeated?.Invoke();
            }
            public override void OnResetReader()
            {
                gunManager.onResetReader?.Invoke();
            }
            public override void ReadTagWithTid(ResultCode error, CommandType action, string epc, string data, float rssi, float phase, float frequency)
            {
                gunManager.readTagWithTid?.Invoke(error, action, epc, data, rssi, phase, frequency);
            }
            public override void ReadTagWithRFM(ResultCode error, CommandType action, string epc, float rssi, float phase, float frequency, string tid, string tempCode, string calibration, string sensorCode)
            {
                gunManager.readTagWithRFM?.Invoke(error, action, epc, rssi, phase, frequency, tid, tempCode, calibration, sensorCode);
            }
        }
        void AsReaderGUNConnected(NSNotification notification)
        {
            mReader = new AsReader(mAsReaderGUN, mAsreaderDelegate);
            if (onAsReaderGUNConnected != null && mReader != null)
            {
                onAsReaderGUNConnected();
            } 
        }

        void AsReaderGUNDisconnected(NSNotification notification)
        {
            if (mReader != null)
            {
                mReader.Disconnect();
                onAsReaderGUNDisconnected?.Invoke();
                mReader = null;
            }
        }
    }
}
