// This file has been autogenerated from a class added in the UI designer.

using System;
using AsReaderGun251GSDK;
using AsRingAccessory251GSDK;
using Foundation;
using UIKit;
using ObjCRuntime;
using CoreAnimation;

namespace AsReaderGun251GDemoApp
{
    public partial class MainViewController : UIViewController
    {
        readonly int                BATTERY_NONE = 0;
        readonly int                BATTERY_QUARTER = 1;
        readonly int                BATTERY_HALF = 2;
        readonly int                BATTERY_THIRD_QUARTER = 3;
        readonly int                BATTERY_FULL = 4;
        readonly int                BATTERY_NODATA = 5;

        readonly int                CONNECTION_OFF = 0;
        readonly int                CONNECTION_ON = 1;
        UIView                      backView;

        readonly AppDelegate        appDelegate = (AppDelegate)UIApplication.SharedApplication.Delegate;
        NSTimer                     batteryCheckTimer;
        UITapGestureRecognizer      logoDebugTap;
        readonly AsRingDeviceSDK    asRingDeviceSDK = AsRingDeviceSDK.SharedAsRingDeviceSDK;
        readonly AsReaderGUNManager gunManager = AsReaderGUNManager.SharedAsReaderGUNManager();
        private AsReader mReader
        {
            get => gunManager.GetCurrentAsReader();
        }

        public MainViewController(IntPtr handle) : base(handle)
        {
        }

        public override void ViewDidLoad()
        {
            base.ViewDidLoad();

            labelDemoVer.Text = appDelegate.GetAppVersion();
            regionLabel.Text = "";
            snLabel.Text = "";
            ufVersion.Text = "";
           
            mainScrollView.ContentSize = new CoreGraphics.CGSize(0, View.Frame.Size.Height);
            mainScrollView.ShowsVerticalScrollIndicator = false;
            logoDebugTap = null;

            //VersionHandler-------
            int buzzerTime = SystemSetting.sharedSystemSetting().getBuzzerTime();
            buzzerTextField.Text = buzzerTime.ToString();
            int vibratorTime = SystemSetting.sharedSystemSetting().getVibratorTime();
            vibratorTextField.Text = vibratorTime.ToString();
            labelSDKVer.Text = gunManager.mAsReaderGUN.AsReaderGUNVersion;
            buzzerTextField.Delegate = new MyTextFieldDelegate(this);
            vibratorTextField.Delegate = new MyTextFieldDelegate(this);
            backView = new UIView
            {
                BackgroundColor = UIColor.Gray,
                Alpha = (nfloat)0.5,
                Frame = new CoreGraphics.CGRect(0, 0, View.Frame.Size.Width, View.Frame.Size.Height)
            };
            mainScrollView.SetContentOffset(new CoreGraphics.CGPoint(0, 0), true);

            UITapGestureRecognizer tapGestureRecognizer = new UITapGestureRecognizer(KeyboardHide);
            tapGestureRecognizer.CancelsTouchesInView = false;
            View.AddGestureRecognizer(tapGestureRecognizer);
            // Perform any additional setup after loading the view, typically from a nib.
        }

        public override void ViewWillAppear(bool animated)
        {
            base.ViewWillAppear(animated);
            gunManager.onAsReaderGUNConnected = () => readerInitializedSetting();

            gunManager.onAsReaderGUNDisconnected = () => DisableAllUIs();

            gunManager.onAsReaderTriggerKeyEvent = (status) => false;

            gunManager.onAsReaderLeftModeKeyEvent = (status) =>
            {
                if (mReader != null)
                {
                    if (mReader.IsRFIDModule && mReader.IsBarcodeModule) return true;
                }
                return false;
            };
            gunManager.onAsReaderRightModeKeyEvent = (status) =>
            {
                if (mReader != null)
                { 
                    if (mReader.IsRFIDModule && mReader.IsBarcodeModule) return true;
                }
                return false;
            };
            gunManager.readerInitialized = (reader) =>
            {
            };
            gunManager.accessResult = (error, action, epc, data, rssi, phase, frequency) =>
            {
                if (error != ResultCode.NoError)
                {
                    UIAlertController alert = UIAlertController.Create("Error", AsResultCode.Msg(error), UIAlertControllerStyle.Alert);
                    UIAlertAction cancelAction = UIAlertAction.Create("OK", UIAlertActionStyle.Cancel, null);
                    alert.AddAction(cancelAction);
                    PresentViewController(alert, true, null);
                }
            };
            gunManager.updateDeviceState = (error) =>
            {
                if (error != ResultCode.NoError)
                {
                    UIAlertController alert = UIAlertController.Create("AsRingDevice", "AsRing device is not available on this device", UIAlertControllerStyle.Alert);
                    UIAlertAction cancelAction = UIAlertAction.Create("OK", UIAlertActionStyle.Cancel, action =>
                    {
                        if (mReader != null)
                            mReader.Disconnect();
                        else
                            DisableAllUIs();
                    });
                    alert.AddAction(cancelAction);
                    PresentViewController(alert, true, null);
                }
            };
        }

        public override void ViewDidAppear(bool animated)
        {
            base.ViewDidAppear(animated);
            //ChangeBatteryImage(BATTERY_NODATA);
            if (NavigationController.RespondsToSelector(new Selector("InteractivePopGestureRecognizer")))
            {
                NavigationController.InteractivePopGestureRecognizer.Enabled = false;
            }
            ReloadData();
        }

        public override void ViewDidDisappear(bool animated)
        {
            base.ViewDidDisappear(animated);
            startMonitoringBattery(false);
        }

        void startMonitoringBattery(bool isStart)
        {
            updateBattery();
            double periodicTime = int.Parse(UserData.getBatteryInterval()) / 1000.0;
            if (batteryCheckTimer != null)
            {
                batteryCheckTimer.Invalidate();
                batteryCheckTimer = null;
            }
            if (isStart)
            {
                batteryCheckTimer = NSTimer.CreateRepeatingScheduledTimer(TimeSpan.FromSeconds(periodicTime), delegate {
                    updateBattery();
                });
            }
        }
        void updateBattery()
        {
            int batteryStatus = 0;
            if (mReader != null)
            {
                batteryStatus = mReader.BatteryStatus;
            }
            ChangeBatteryImage(batteryStatus);
        }

        partial void OnClickVibratorButton(NSObject sender)
        {
            int vibrator = int.Parse(vibratorTextField.Text);

            if (vibrator > 0 && vibrator < 5000)
            {
                SystemSetting.sharedSystemSetting().setVibratorTime(vibrator);
                _ = mReader.StartVibratorWithVibratorTime(vibrator);
            }
            else
            {
                UIAlertController alert = UIAlertController.Create("Vibrator Value", "Please enter the value (1~4999)", UIAlertControllerStyle.Alert);
                UIAlertAction cancelAction = UIAlertAction.Create("OK", UIAlertActionStyle.Cancel, null);
                alert.AddAction(cancelAction);
                PresentViewController(alert, true, null);
            }
        }

        partial void OnClickBuzzerButton(NSObject sender)
        {
            int buzzerTime = int.Parse(buzzerTextField.Text);

            if (buzzerTime > 0 && buzzerTime < 5000)
            {
                SystemSetting.sharedSystemSetting().setBuzzerTime(buzzerTime);
                _ = mReader.StartBuzzerWithBuzzerTime(buzzerTime);
            }
            else
            {
                UIAlertController alert = UIAlertController.Create("Buzzer Value", "Please enter the value (1~4999)", UIAlertControllerStyle.Alert);
                UIAlertAction cancelAction = UIAlertAction.Create("OK", UIAlertActionStyle.Cancel, null);
                alert.AddAction(cancelAction);
                PresentViewController(alert, true, null);
            }
        }

        partial void BtnModeLedTestActon(NSObject _)
        {
            UIAlertController alertModeLedTest = UIAlertController.Create("MODE LED TEST", "\n\n\n\n\n\n", UIAlertControllerStyle.Alert);
            UIAlertAction cancelAction = UIAlertAction.Create("OK", UIAlertActionStyle.Cancel, null);
            alertModeLedTest.AddAction(cancelAction);

            UILabel labelLeft = new UILabel(new CoreGraphics.CGRect(5, 50, 200, 50));
            UILabel labelRight = new UILabel(new CoreGraphics.CGRect(5, 100, 200, 50));
            labelLeft.Text = " LEFT LED (ON/OFF)";
            labelRight.Text = " RIGHT LED (ON/OFF)";

            UISwitch switchLeft = new UISwitch(new CoreGraphics.CGRect(200, 60, 0, 0));
            switchLeft.Tag = 1;
            switchLeft.AddTarget(this, new Selector("changeModeLedTestAction:"), UIControlEvent.ValueChanged);
            switchLeft.On = mReader.ModeLeftLedOn;

            UISwitch switchRight = new UISwitch(new CoreGraphics.CGRect(200, 110, 0, 0));
            switchRight.Tag = 2;
            switchRight.AddTarget(this, new Selector("changeModeLedTestAction:"), UIControlEvent.ValueChanged);
            switchRight.On = mReader.ModeRightLedOn;

            alertModeLedTest.View.AddSubview(labelLeft);
            alertModeLedTest.View.AddSubview(labelRight);
            alertModeLedTest.View.AddSubview(switchLeft);
            alertModeLedTest.View.AddSubview(switchRight);

            PresentViewController(alertModeLedTest, true, null);
        }

        [Export("changeModeLedTestAction:")]
        public void ChangeModeLedTestAction(UISwitch sender)
        {
            nint tagID = sender.Tag;
            switch (tagID)
            {
                case 1:
                    mReader.ModeLeftLedOn = sender.On;
                    break;
                case 2:
                    mReader.ModeRightLedOn = sender.On;
                    break;
            }
        }

        void ReloadData()
        {

            if (mReader != null)
            {
                asRingPlusVersion.Text = asRingDeviceSDK.AsRingFirmwareRevision;
                labelFirmwareVer.Text = mReader.FirmwareVersion;
                labelAddress.Text = mReader.MAsReaderGUN.Address;
                EnableAllButtons();

                if (!mReader.IsBarcodeModule)
                    DisableBarcodeButtons();

                if (!mReader.IsRFIDModule)
                    DisableRFIDButtons();

                regionLabel.Text = mReader.RegionName;
                snLabel.Text = mReader.SerialNumber;
                ufVersion.Text = mReader.RFModuleVersion;
                int batteryStatus = mReader.BatteryStatus;
                ChangeBatteryImage(batteryStatus);
                SystemSetting.sharedSystemSetting().asReaderGUNSetting();
                AppDelegate.dismissGlobalHUD();
                mReader.SetScanMode(ScanMode.RFIDScanMode);
            }
            else
                DisableAllUIs();
        }

//#pragma mark - Custom
        void EnableAllButtons()
        {
            foreach (MainButton btn in buttonCollection)
            {
                btn.EnableButton();
            }
            startMonitoringBattery(true);
            ChangeLogoImage(CONNECTION_ON);
            buzzerButton.SetTitleColor(UIColor.Black, UIControlState.Normal);
            vibratorButton.SetTitleColor(UIColor.Black, UIControlState.Normal);
            buzzerTextField.Text = SystemSetting.sharedSystemSetting().getBuzzerTime().ToString();
            vibratorTextField.Text = SystemSetting.sharedSystemSetting().getVibratorTime().ToString();
            buzzerTextField.Enabled = true;
            vibratorTextField.Enabled = true;
            buzzerButton.Enabled = true;
            vibratorButton.Enabled = true;
            ledTestButton.Enabled = true;
        }

        void DisableAllUIs()
        {
            startMonitoringBattery(false);
            KeyboardHide();
            mainScrollView.SetContentOffset(new CoreGraphics.CGPoint(0, 0), true);

            asRingPlusVersion.Text = "";
            labelFirmwareVer.Text = "";
            labelAddress.Text = "";
            regionLabel.Text = "";
            snLabel.Text = "";
            ufVersion.Text = "";

            buzzerButton.SetTitleColor(UIColor.Gray, UIControlState.Normal);
            vibratorButton.SetTitleColor(UIColor.Gray, UIControlState.Normal);
            buzzerTextField.Text = SystemSetting.sharedSystemSetting().getBuzzerTime().ToString();
            vibratorTextField.Text = SystemSetting.sharedSystemSetting().getVibratorTime().ToString();
            buzzerTextField.Enabled = false;
            vibratorTextField.Enabled = false;
            buzzerButton.Enabled = false;
            vibratorButton.Enabled = false;
            ledTestButton.Enabled = false;

            foreach (MainButton btn in buttonCollection)
            {
                btn.DisableButton();
            }
            ChangeBatteryImage(BATTERY_NODATA);
            ChangeLogoImage(CONNECTION_OFF);
        }

        void DisableBarcodeButtons()
        {
            buttonCollection[6].DisableButton();
            buttonCollection[7].DisableButton();
        }

        void DisableRFIDButtons()
        {
            buttonCollection[0].DisableButton();
            buttonCollection[1].DisableButton();
            buttonCollection[2].DisableButton();
            buttonCollection[3].DisableButton();
            buttonCollection[4].DisableButton();
            buttonCollection[5].DisableButton();
            buttonCollection[8].DisableButton();
        }

        void ChangeBatteryImage(int batteryStatus)
        {
            if (batteryStatus == BATTERY_NONE) batteryImage.Image = UIImage.FromBundle("battery0");
            else if (batteryStatus == BATTERY_QUARTER) batteryImage.Image = UIImage.FromBundle("battery1");
            else if (batteryStatus == BATTERY_HALF) batteryImage.Image = UIImage.FromBundle("battery2");
            else if (batteryStatus == BATTERY_THIRD_QUARTER) batteryImage.Image = UIImage.FromBundle("battery3");
            else if (batteryStatus == BATTERY_FULL) batteryImage.Image = UIImage.FromBundle("battery4");
            else if (batteryStatus == BATTERY_NODATA) batteryImage.Image = UIImage.FromBundle("battery");
        }

        void ChangeLogoImage(int connectionStatus)
        {
            if (connectionStatus == CONNECTION_OFF)
            {
                logoImage.Image = UIImage.FromBundle("asreader_logo_off");

                if (logoDebugTap != null)
                {
                    logoImage.UserInteractionEnabled = false;
                    logoImage.RemoveGestureRecognizer(logoDebugTap);
                    logoDebugTap = null;
                }
            }
            else if (connectionStatus == CONNECTION_ON)
            {
                logoImage.Image = UIImage.FromBundle("asreader_logo_on");
                if (logoDebugTap != null)
                {
                    logoDebugTap = null;
                }
                logoDebugTap = new UITapGestureRecognizer(LogoDebugTapDetected)
                {
                    NumberOfTapsRequired = 5
                };
                logoImage.UserInteractionEnabled = true;
                logoImage.AddGestureRecognizer(logoDebugTap);
            }
        }

        void LogoDebugTapDetected(UITapGestureRecognizer sender)
        {
            if (sender.State == UIGestureRecognizerState.Ended)
            {
                UIAlertController alert = UIAlertController.Create("Please select the operationg mode.", null, UIAlertControllerStyle.ActionSheet);
                UIAlertAction debugAction = UIAlertAction.Create("Debug Mode", UIAlertActionStyle.Default, Action =>
                {
                    mReader.SetDebugMode(true);
                });
                UIAlertAction normalAction = UIAlertAction.Create("Normal Mode", UIAlertActionStyle.Default, Action =>
                {
                    mReader.SetDebugMode(false);
                });

                alert.AddAction(debugAction);
                alert.AddAction(normalAction);
                PresentViewController(alert, true, null);
            }
        }

        private class MyTextFieldDelegate : UITextFieldDelegate
        {
            MainViewController currentViewContorller;
            public MyTextFieldDelegate(MainViewController controller)
            {
                currentViewContorller = controller;
            }

            public override void EditingStarted(UITextField textField)
            {
                currentViewContorller.mainScrollView.ScrollEnabled = false;
                currentViewContorller.View.AddSubview(currentViewContorller.backView);
                if (currentViewContorller.buzzerTextField.IsFirstResponder)
                {
                    currentViewContorller.buzzerTextField.KeyboardType = UIKeyboardType.NumberPad;
                }
                else if (currentViewContorller.vibratorTextField.IsFirstResponder)
                {
                    currentViewContorller.vibratorTextField.KeyboardType = UIKeyboardType.NumberPad;
                }
                currentViewContorller.mainScrollView.SetContentOffset(new CoreGraphics.CGPoint(0, 260), true);
            }

            public override bool ShouldChangeCharacters(UITextField textField, NSRange range, string replacementString)
            {
                if (int.Parse(textField.Text + replacementString) <= 0)
                {
                    textField.Text = "1";
                }

                return true;
            }
        }

        void KeyboardHide()
        {
            mainScrollView.ScrollEnabled = true;
            backView.RemoveFromSuperview();
            vibratorTextField.ResignFirstResponder();
            buzzerTextField.ResignFirstResponder();
            if (buzzerTextField.Text == "") buzzerTextField.Text = "0";
            if (int.Parse(buzzerTextField.Text) == 0)
            {
                buzzerTextField.Text = "1";
            }
            if (int.Parse(vibratorTextField.Text) == 0)
            {
                vibratorTextField.Text = "1";
            }

            if (int.Parse(buzzerTextField.Text) > 4999)
            {
                UIAlertController alert = UIAlertController.Create("Buzzer Value", "Please enter the value (1~4999)", UIAlertControllerStyle.Alert);
                UIAlertAction cancelAction = UIAlertAction.Create("OK", UIAlertActionStyle.Cancel, null);
                alert.AddAction(cancelAction);
                PresentViewController(alert, true, null);
                buzzerTextField.Text = "4999";
            }
            if (int.Parse(vibratorTextField.Text) > 4999)
            {
                UIAlertController alert = UIAlertController.Create("Vibrator Value", "Please enter the value (1~4999)", UIAlertControllerStyle.Alert);
                UIAlertAction cancelAction = UIAlertAction.Create("OK", UIAlertActionStyle.Cancel, null);
                alert.AddAction(cancelAction);
                PresentViewController(alert, true, null);
                vibratorTextField.Text = "4999";
            }


            SystemSetting.sharedSystemSetting().setVibratorTime(int.Parse(vibratorTextField.Text));
            SystemSetting.sharedSystemSetting().setBuzzerTime(int.Parse(buzzerTextField.Text));
            mainScrollView.SetContentOffset(new CoreGraphics.CGPoint(0, 100), true);
        }
        
        void readerInitializedSetting()
        {
            Console.WriteLine("Main.readerInitializedSetting");
            EnableAllButtons();
            if (!mReader.IsBarcodeModule)
                DisableBarcodeButtons();
            else
            {
                AppDelegate.showGlobalProgressHUDWithTitle("Barcode Setting...");
                CATransaction.Commit();
                SystemSetting.sharedSystemSetting().barcodeSetting();
                AppDelegate.dismissGlobalHUD();
                CATransaction.Commit();
            }
            if (!mReader.IsRFIDModule)
                DisableRFIDButtons();
            else
            {
                AppDelegate.showGlobalProgressHUDWithTitle("RFID Setting...");
                CATransaction.Commit();
                SystemSetting.sharedSystemSetting().RFIDSetting();
                AppDelegate.dismissGlobalHUD();
                CATransaction.Commit();
            }
            ReloadData();
        }
    }
}
